==> prometheus_tools_abm-0.1.51455.tar.gz: prometheus_tools_abm-0.1.51455/prometheus_tools/collectors/incoming_http_requests_metrics_collector.py <==
from dataclasses import dataclass, field
from typing import List, Union

from aiohttp import web
from aiohttp.web_response import Response
from http_tools.error_middleware import Handler
from http_tools.http_server import WsHandler
from prometheus_client import CollectorRegistry, REGISTRY, Counter, Gauge, Histogram
from prometheus_client.utils import INF
from ..tools.timer import Timer


class IncomingHttpRequestsMetricsCollector:
    class Label:
        method = "method"
        path = "path"
        status_code = "status_code"

    @dataclass
    class Context:
        registry: CollectorRegistry = REGISTRY

    @dataclass
    class Config:
        exclude_paths: List[str] = field(default_factory=lambda: ["/metrics"])
        latency_buckets: List[float] = field(default_factory=lambda: [.1, .25, .5, .75, 1.0, 2.5, 5, 7.5, 10.0, INF])

    def __init__(self, config: Config, context: Context) -> None:
        self.config = config
        self.context = context

        self.total_requests_counter = Counter(
            "total_incoming_requests",
            "Total number of HTTP requests received",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
        )

        self.active_requests_gauge = Gauge(
            "active_incoming_requests",
            "Number of active HTTP requests",
            labelnames=[self.Label.method, self.Label.path],
            registry=self.context.registry
        )

        self.requests_latency_histogram = Histogram(
            "incoming_requests_latency",
            "Http request latency",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
            buckets=self.config.latency_buckets
        )

        self.network_traffic_counter = Counter(
            "network_traffic",
            "Network traffic in bytes",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
        )

    def create_middleware(self) -> web.middleware:
        @web.middleware
        async def middleware(request: web.Request, handler: Union[Handler, WsHandler], *args, **kwargs) -> Response:
            if request.path in self.config.exclude_paths:
                return await handler(request)

            labels = {self.Label.method: request.method, self.Label.path: request.path}
            timer = Timer()

            with timer, self.active_requests_gauge.labels(**labels).track_inprogress():
                response: Response = await handler(request)

            labels[self.Label.status_code] = response.status
            self.requests_latency_histogram.labels(**labels).observe(timer.duration)
            self.total_requests_counter.labels(**labels).inc()
            self.network_traffic_counter.labels(**labels).inc(response.content_length)

            return response

        return middleware

==> prometheus_tools_abm-0.1.53370.tar.gz: prometheus_tools_abm-0.1.53370/prometheus_tools/collectors/incoming_http_requests_metrics_collector.py <==
from dataclasses import dataclass, field
from typing import List, Union

from aiohttp import web
from aiohttp.web_response import Response
from http_tools.error_middleware import Handler
from http_tools.http_server import WsHandler
from prometheus_client import CollectorRegistry, REGISTRY, Counter, Gauge, Histogram
from prometheus_client.utils import INF
from ..tools.timer import Timer


class IncomingHttpRequestsMetricsCollector:
    class Label:
        method = "method"
        path = "path"
        status_code = "status_code"

    @dataclass
    class Context:
        registry: CollectorRegistry = REGISTRY

    @dataclass
    class Config:
        exclude_paths: List[str] = field(default_factory=lambda: ["/metrics"])
        latency_buckets: List[float] = field(
            default_factory=lambda: [.05, .1, .25, .5, .75, 1.0, 2.5, 5, 7.5, 10.0, 30.0, 60.0, INF]
        )

    def __init__(self, config: Config, context: Context) -> None:
        self.config = config
        self.context = context

        self.total_requests_counter = Counter(
            "total_incoming_requests",
            "Total number of HTTP requests received",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
        )

        self.active_requests_gauge = Gauge(
            "active_incoming_requests",
            "Number of active HTTP requests",
            labelnames=[self.Label.method, self.Label.path],
            registry=self.context.registry
        )

        self.requests_latency_histogram = Histogram(
            "incoming_requests_latency",
            "Http request latency",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
            buckets=self.config.latency_buckets
        )

        self.network_traffic_counter = Counter(
            "network_traffic",
            "Network traffic in bytes",
            labelnames=[self.Label.method, self.Label.path, self.Label.status_code],
            registry=self.context.registry,
        )

    def create_middleware(self) -> web.middleware:
        @web.middleware
        async def middleware(request: web.Request, handler: Union[Handler, WsHandler], *args, **kwargs) -> Response:
            if request.path in self.config.exclude_paths:
                return await handler(request)

            labels = {self.Label.method: request.method, self.Label.path: request.path}
            timer = Timer()

            self.total_requests_counter.labels(**labels).inc()
            with timer, self.active_requests_gauge.labels(**labels).track_inprogress():
                response: Response = await handler(request)

            labels[self.Label.status_code] = response.status
            self.requests_latency_histogram.labels(**labels).observe(timer.duration)
            self.network_traffic_counter.labels(**labels).inc(response.content_length)

            return response

        return middleware
